from argparse import ArgumentParser
from typing import Optional

from satoricli.api import client
from satoricli.cli.utils import autoformat, console

from .base import BaseCommand
from .report import ReportCommand


class ReportsCommand(BaseCommand):
    name = "reports"

    def register_args(self, parser: ArgumentParser):
        parser.add_argument("-p", "--page", type=int, default=1)
        parser.add_argument("-l", "--limit", type=int, default=20)
        parser.add_argument("-f", "--filter")

    def __call__(self, page: int, limit: int, filter: Optional[str], **kwargs):
        res = client.get(
            "/reports", params={"page": page, "limit": limit, "filter": filter}
        ).json()

        if not kwargs["json"]:
            if res["count"] == 0:
                console.print("No reports found")
                return

            ReportCommand.print_report_list(res["list"])
        elif kwargs["json"]:
            autoformat(res, jsonfmt=kwargs["json"])
        else:  # single report
            ReportCommand.print_report_single(res)



from argparse import ArgumentParser

from satoricli.api import client
from satoricli.cli.utils import autotable, BootstrapTable, autoformat

from satoricli.playbooks import display_public_playbooks

from .base import BaseCommand
from ..utils import get_offset


class PlaybooksCommand(BaseCommand):
    name = "playbooks"

    def register_args(self, parser: ArgumentParser):
        parser.add_argument(
            "-p",
            "--page",
            dest="page",
            type=int,
            default=1,
            help="Playbooks page number",
        )
        parser.add_argument(
            "-l",
            "--limit",
            dest="limit",
            type=int,
            default=20,
            help="Page limit number",
        )
        parser.add_argument(
            "--public", action="store_true", help="Fetch public satori playbooks"
        )

    def __call__(self, page: int, limit: int, public: bool, **kwargs):
        if public:
            display_public_playbooks()
            return
        offset = get_offset(page, limit)
        data = client.get(
            "/playbooks", params={"offset": offset, "limit": limit}
        ).json()

        if not kwargs["json"]:
            autotable(BootstrapTable(**data), limit=limit, page=page)
        else:
            autoformat(data["rows"], jsonfmt=kwargs["json"], list_separator="-" * 48)



from argparse import ArgumentParser
from typing import Literal, Optional

from satoricli.api import client
from satoricli.cli.utils import autoformat, autotable, console

from .base import BaseCommand
from .dashboard import DashboardCommand
from .report import ReportCommand
from ..utils import BootstrapTable, get_offset


class TeamCommand(BaseCommand):
    name = "team"

    def register_args(self, parser: ArgumentParser):
        parser.add_argument("--role", default="READ", help="User role")

        # Use with the add command
        parser.add_argument("--repo", help="Repo name")
        parser.add_argument("--email", help="User email")
        parser.add_argument("--github", help="Github user name")
        parser.add_argument("--monitor", help="Monitor id")

        parser.add_argument("id")
        parser.add_argument(
            "action",
            nargs="?",
            choices=(
                "show",
                "create",
                "members",
                "repos",
                "monitors",
                "reports",
                "get_config",
                "set_config",
                "get_token",
                "refresh_token",
                "del",
                "delete",
                "add",
                "settings",
                "playbooks",
            ),
            default="show",
        )
        parser.add_argument("config_name", nargs="?")
        parser.add_argument("config_value", nargs="?")
        parser.add_argument("-p", "--page", type=int, default=1)
        parser.add_argument("-l", "--limit", type=int, default=20)

    def __call__(
        self,
        id: str,
        action: Literal[
            "show",
            "create",
            "members",
            "repos",
            "monitors",
            "reports",
            "get_config",
            "set_config",
            "get_token",
            "refresh_token",
            "del",
            "delete",
            "add",
            "settings",
            "playbooks",
        ],
        role: Optional[str],
        repo: Optional[str],
        email: Optional[str],
        github: Optional[str],
        monitor: Optional[str],
        config_name: Optional[str],
        config_value: Optional[str],
        page: int,
        limit: int,
        **kwargs,
    ):
        if action == "show":
            info = client.get(f"/teams/{id}").json()
            return DashboardCommand.generate_dashboard(info)
        elif action == "create":
            info = client.post(f"/teams/{id}").json()
        elif action == "members":
            info = client.get(f"/teams/{id}/members").json()
        elif action == "add":
            if email:
                info = client.post(
                    f"/teams/{id}/members", json={"email": email, "role": role}
                ).json()
            elif github:
                info = client.post(
                    f"/teams/{id}/members", json={"github": github, "role": role}
                ).json()
            elif repo:
                info = client.post(f"/teams/{id}/repos", json={"repo": repo}).json()
            elif monitor:
                info = client.post(
                    f"/teams/{id}/monitors", json={"monitor": monitor}
                ).json()
            else:
                raise Exception("Use --github, --email, --repo or --monitor")
        elif action == "repos":
            offset = get_offset(page, limit)
            info = client.get(
                f"/teams/{id}/repos", params={"offset": offset, "limit": limit}
            ).json()
            autotable(
                BootstrapTable(**info), "b blue", False, (20, 20, None), page, limit
            )
            return
        elif action == "get_config":
            info = client.get(f"/teams/{id}/config/{config_name}").json()
        elif action == "set_config":
            info = client.put(
                f"/teams/{id}/config",
                json={"name": config_name, "value": config_value},
            ).json()
        elif action == "get_token":
            info = client.get(f"/teams/{id}/token").json()
        elif action == "refresh_token":
            info = client.put(f"/teams/{id}/token").json()
        elif action == "del":
            if email:
                client.request("DELETE", f"/teams/{id}/members", json={"email": email})
                console.print("Team member deleted")
            elif repo:
                client.request("DELETE", f"/teams/{id}/repos", json={"repo": repo})
                console.print("Team repo deleted")
            elif github:
                client.request(
                    "DELETE", f"/teams/{id}/members", json={"github": github}
                )
                console.print("Team member deleted")
            else:
                raise Exception("Use --github, --email or --repo")
            return
        elif action == "delete":
            client.delete(f"/teams/{id}")
            console.print("Team deleted")
            return
        elif action == "monitors":
            info = client.get(f"/teams/{id}/monitors").json()
            if not kwargs["json"]:
                autotable(info["rows"], "b blue", widths=(20, 20, None))
                return
        elif action == "reports":
            info = client.get(f"/teams/{id}/reports").json()
            return ReportCommand.print_report_list(info["list"])
        elif action == "settings":
            info = client.get(f"/teams/{id}/config").json()
        elif action == "playbooks":
            info = client.get(f"/teams/{id}/playbooks").json()
            if not kwargs["json"]:
                autotable(info, "b blue")
                return

        autoformat(info, jsonfmt=kwargs["json"], list_separator="*" * 48, table=True)



import json
from argparse import ArgumentParser
from datetime import date
from pathlib import Path
from typing import Literal, Optional

from rich.live import Live
from satoricli.api import WS_HOST, client
from satoricli.cli.utils import autoformat, console
from websocket import WebSocketApp

from ..arguments import date_args
from .base import BaseCommand


class ScanCommand(BaseCommand):
    name = "scan"
    options = (date_args,)

    def register_args(self, parser: ArgumentParser):
        parser.add_argument("repository", metavar="REPOSITORY")
        parser.add_argument(
            "action",
            metavar="ACTION",
            choices=(
                "new",
                "stop",
                "status",
                "clean",
                "check-forks",
                "check-commits",
            ),
            nargs="?",
            default="new",
            help="action to perform",
        )
        parser.add_argument(
            "-c", "--coverage", type=float, default=1.0, help="coverage"
        )
        parser.add_argument("--skip-check", action="store_true")
        parser.add_argument("--delete-commits", action="store_true")
        parser.add_argument("-s", "--sync", action="store_true")
        parser.add_argument("-d", "--data", help="Secrets", default="")
        parser.add_argument("-b", "--branch", default="main", help="Repo branch")
        parser.add_argument("--filter", help="Filter names")
        parser.add_argument("--playbook", help="Playbook url or file")

    def __call__(
        self,
        repository: str,
        action: Literal[
            "new",
            "stop",
            "status",
            "clean",
            "check-forks",
            "check-commits",
        ],
        coverage: float,
        sync: bool,
        delete_commits: bool,
        data: Optional[str],
        branch: str,
        from_date: Optional[date],
        to_date: Optional[date],
        playbook: Optional[str],
        **kwargs,
    ):
        if action == "new":
            config = None

            if playbook:
                if (path := Path(playbook)) and path.is_file():
                    config = path.read_text()

                if playbook.startswith("satori://"):
                    config = playbook

            info = client.get(
                "/repos/scan",
                params={
                    "url": repository,
                    "data": data,
                    "playbook": config,
                    "coverage": coverage,
                    "from": from_date,
                    "to": to_date,
                    "branch": branch,
                },
            ).json()
            if sync:
                return self.scan_sync(repository)
        elif action == "clean":
            info = client.get(
                f"/repos/{repository}/clean", params={"delete_commits": delete_commits}
            ).json()
        elif action == "stop":
            info = client.get(f"/repos/scan/{repository}/stop").json()
        elif action == "status":
            if sync:
                return self.scan_sync(repository)
            else:
                info = client.get(f"/repos/scan/{repository}/status").json()
        elif action == "check-forks":
            info = client.get(f"/repos/scan/{repository}/check-forks").json()
        elif action == "check-commits":
            console.print(f"Checking the list of commits of the repo {repository}")
            info = client.get(
                f"/repos/scan/{repository}/check-commits", params={"branch": branch}
            ).json()
            if sync:
                return ScanCommand.scan_sync(repository)
        autoformat(info, jsonfmt=kwargs["json"], list_separator="-" * 48)

    @staticmethod
    def scan_sync(id):
        live = Live("Loading data...", console=console, auto_refresh=False)

        def on_open(app: WebSocketApp):
            app.send(json.dumps({"action": "scan-status", "id": id}))

        def on_message(app: WebSocketApp, message):
            print(message)
            try:
                # try to load the message as a json
                stats = json.loads(message)
                # make text readable
                output = autoformat(stats, echo=False)
            except Exception:
                # if fail print plain text
                live.update(message)
            else:
                if output:
                    live.update(output)
            finally:
                live.refresh()

        def on_error(app: WebSocketApp, error):
            console.print("[error]Error:[/] " + str(error))
            app.has_errored = True  # ?????

        def on_close(app: WebSocketApp, close_status_code: int, close_msg: str):
            if close_status_code != 1000:
                on_error(app, close_msg)  # ?????

        app = WebSocketApp(
            WS_HOST,
            on_open=on_open,
            on_message=on_message,
            on_close=on_close,
            on_error=on_error,
            header={"Authorization": client.headers["Authorization"]},
        )
        app.run_forever()

        if app.has_errored:
            return 1
